
/*******************************************************************************
* Copyright (c) 2015 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
*
* This file is part of PAPPSOms-tools.
*
*     PAPPSOms-tools is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     PAPPSOms-tools is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with PAPPSOms-tools.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial API and implementation
******************************************************************************/

#include "spectrumwindow.h"
#include <QFileDialog>
#include <QSettings>
#include <QSvgGenerator>
#include <QDebug>
#include <pappsomspp/peptide/peptidestrparser.h>
#include "ui_spectrumwindow.h"


SpectrumWindow::SpectrumWindow(QWidget *parent):
    QMainWindow(parent),
    ui(new Ui::SpectrumWindow)
{
    ui->setupUi(this);

    _p_spectrum_overlay = new LoadingOverlay(ui->spectrumWidget);

    _worker_loader = new PwizLoaderThread();
    _worker_loader->moveToThread(&workerThread);
#if QT_VERSION >= 0x050000
    // Qt5 code
    connect(&workerThread, &QThread::finished, _worker_loader, &QObject::deleteLater);
    connect(this, &SpectrumWindow::operateMsDataFile, _worker_loader, &PwizLoaderThread::doMsDataFileLoad);
    connect(_worker_loader, &PwizLoaderThread::msDataReady, this, &SpectrumWindow::handleMsDataFile);
    connect (_p_spectrum_overlay,&LoadingOverlay::mzChanged, this, &SpectrumWindow::setMz);

#else
// Qt4 code
    connect(&workerThread, SIGNAL(finished()), _worker_loader, SLOT(deleteLater()));
    connect(this, SIGNAL(operateMsDataFile(QString)), _worker_loader,SLOT(doMsDataFileLoad(QString)));
    connect(_worker_loader, SIGNAL(msDataReady(pwiz::msdata::MSDataFile *)), this, SLOT(handleMsDataFile(pwiz::msdata::MSDataFile *)));

    connect (_p_spectrum_overlay,SIGNAL(mzChanged(double)), this, SLOT(setMz(double)));

    connect(ui->actionLoadMzFile, SIGNAL(triggered()), this, SLOT(selectMzFile()));
    connect(ui->actionSaveSvg, SIGNAL(triggered()), this, SLOT(saveSvgFile()));
#endif
    /*
    */
    workerThread.start();


}

SpectrumWindow::~SpectrumWindow()
{
    workerThread.quit();
    workerThread.wait();
    if (_p_ms_data_file != nullptr) delete _p_ms_data_file;
    delete ui;

}

void SpectrumWindow::setPeptideSp(pappso::PeptideSp peptide_sp) {
    qDebug() << "SpectrumWindow::setPeptideSp";
    ui->spectrumWidget->setPeptideSp(peptide_sp);
}


void SpectrumWindow::setPrecision(pappso::PrecisionP p_precision) {
    qDebug() << "SpectrumWindow::setPrecision";
    ui->spectrumWidget->setPrecision(p_precision);
}


void SpectrumWindow::setFilteredSpectrumSp(SpectrumSp filtered_spectrum) {
    qDebug() << "SpectrumWindow::setFilteredSpectrumSp";
    QualifiedSpectrum spectrum_copy(_spectrum);
    spectrum_copy.setOriginalSpectrumSp(filtered_spectrum);
    ui->spectrumWidget->setQualifiedSpectrum(spectrum_copy);
}

void SpectrumWindow::setMz(double mz) {
    if (mz >0 ) {
        ui->statusbar->showMessage(QString("mz=%1").arg(QString::number(mz, 'g', 10)));
    }
    else {
        ui->statusbar->showMessage("");
    }
    emit mzChanged(mz);
}

void SpectrumWindow::handleMsDataFile(pwiz::msdata::MSDataFile * p_ms_data_file) {
    qDebug() << "handleMsDataFile begin";

    QSettings settings;
    if (_p_ms_data_file != nullptr) delete _p_ms_data_file;
    _p_ms_data_file = p_ms_data_file;
    settings.setValue("path/mzfile", _mz_file_info.absolutePath());
    ui->mzFileName->setText(_mz_file_info.absoluteFilePath());
    _p_spectrum_overlay->hideLoadingMessage();

    _msrun_id.setFilename( _mz_file_info.absoluteFilePath());


    try {

        _spectrum = getQualifiedSpectrumFromPwizMSDataFile(_p_ms_data_file, ui->scanNumSpinBox->value());
        SpectrumId spectrum_id = _spectrum.getSpectrumId();
        spectrum_id.setMsRunId(_msrun_id);
        _spectrum.setSpectrumId(spectrum_id);
        emit spectrumChanged (_spectrum);
    }
    catch (PappsoException exception) {
    }


}

void SpectrumWindow::setScanNum(int scan_num) {
    _p_ms_data_file, ui->scanNumSpinBox->setValue(scan_num);
    if (_p_ms_data_file == nullptr) {
        return;
    }
    if (_worker_loader->isLoading())  {
        return;
    }
    try {
        _spectrum = getQualifiedSpectrumFromPwizMSDataFile(_p_ms_data_file, scan_num);

        SpectrumId spectrum_id = _spectrum.getSpectrumId();
        spectrum_id.setMsRunId(_msrun_id);
        _spectrum.setSpectrumId(spectrum_id);
        emit spectrumChanged (_spectrum);
    }
    catch (PappsoException exception) {
    }
}

void SpectrumWindow::selectMzFile() {

    if (_worker_loader->isLoading()) return;
    QSettings settings;
    QString default_location = settings.value("path/mzfile", "").toString();

    QString filename = QFileDialog::getOpenFileName(this,
                       tr("Open mz File"), default_location,
                       tr("mzXML files (*.mzXML);;mzML files (*.mzML);;MGF files (*.mgf);;all files (*)"));

    if (filename.isEmpty()) {
        return;
    }
    setMzDataFile(filename);
}

void SpectrumWindow::setMzDataFile(const QString & filename) {

    QFileInfo new_mz_file;
    new_mz_file.setFile(filename);
    if (new_mz_file != _mz_file_info) {
        _mz_file_info.setFile(filename);

        ui->mzFileName->setText(tr("loading..."));
        _p_spectrum_overlay->displayLoadingMessage();
        emit operateMsDataFile(_mz_file_info.absoluteFilePath());
        //_p_ms_data_file = getPwizMSDataFile(_mz_file_info.absoluteFilePath());
    }
}

void SpectrumWindow::saveSvgFile() {
    QSettings settings;
    QString default_location = settings.value("path/save", "").toString();

    QString filename = QFileDialog::getSaveFileName(this,
                       tr("Save SVG File"), default_location,
                       tr("SVG files (*.svg);;all files (*)"));

    QSvgGenerator generator;
    //generator.setOutputDevice(&buffer);
    generator.setFileName(filename);
    generator.setSize(QSize(1200, 500));
    generator.setViewBox(QRect(0, 0, 1200, 500));
    generator.setTitle(tr("PAPPSO MS tools SVG spectrum generator"));
    generator.setDescription(tr("This is an annotated SVG spectrum"));
    QPainter painter;
    painter.begin(&generator);

    ui->spectrumWidget->paint(painter);
    painter.end();


}

void SpectrumWindow::setIsotopeMassList(std::vector<pappso::PeptideNaturalIsotopeAverageSp> isotopeMassList) {
    ui->spectrumWidget->setIsotopeMassList(isotopeMassList);
}
